<?php

namespace App\Models;

use DateTime;
use Illuminate\Database\Eloquent\Concerns\HasTimestamps;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Support\Collection;

/**
 * Class Evaluation.
 *
 * @property int                                                             $id
 * @property DateTime                                                        $createdAt
 * @property DateTime                                                        $updatedAt
 * @property DateTime                                                        $updated_at
 * @property DateTime                                                        $created_at
 * @property DangerLevelEvaluation[]|Collection<DangerLevelEvaluation>       $dangerLevels
 * @property DangerLevelEvaluation[]|Collection<DangerLevelEvaluation>       $danger_levels
 * @property MeasureLevel[]|Collection<MeasureLevel>                         $measureLevels
 * @property MeasureLevel[]|Collection<MeasureLevel>                         $measure_levels
 * @property EvaluationMaturityAnswer[]|Collection<EvaluationMaturityAnswer> $maturityLevels
 * @property EvaluationMaturityAnswer[]|Collection<EvaluationMaturityAnswer> $maturity_levels
 * @property int                                                             $status
 * @property Organization                                                    $organization
 * @property int                                                             $organization_id
 * @property int                                                             $current_step
 * @property int                                                             $current_step_status
 * @property string                                                          $author
 * @property string                                                          $updated_by
 */
class Evaluation extends Model
{
    use HasFactory;
    use HasTimestamps;

    public const STATUS_ONGOING = 1;
    public const STATUS_DONE = 2;

    public const STEP_DANGERS = 1;
    public const STEP_MEASURES = 2;
    public const STEP_RECAP = 3;
    public const STEP_ACTIONS = 4;
    public const STEP_MATURITY = 5;
    public const STEP_RESULTS = 6;

    public const STATUS = [
        self::STATUS_ONGOING => 'En cours',
        self::STATUS_DONE => 'Terminé',
    ];

    public function dangerLevels(): HasMany
    {
        return $this->hasMany(DangerLevelEvaluation::class);
    }

    public function measureLevels(): HasMany
    {
        return $this->hasMany(MeasureLevel::class);
    }

    public function organization(): BelongsTo
    {
        return $this->belongsTo(Organization::class);
    }

    public function maturityLevels(): HasMany
    {
        return $this->hasMany(EvaluationMaturityAnswer::class);
    }

    public function getResidualDangers()
    {
        // Get all scenarios
        // For each scenario
        // For each measure
        // check if level is zero
        // if all measures are 0 for this scenario then it is a residual risk

        $scenarios = Scenario::with(['dangers', 'measures'])->get();

        $residual_scenarios = $scenarios;

        foreach ($scenarios as $scenario) {
            foreach ($scenario->measures as $measure) {
                // Get the level of this measure for this evaluation
                $level = $this->measureLevels->first(function (MeasureLevel $ml) use ($measure) {
                    return $ml->measure_id === $measure->id;
                });
                if ($level->actual_level > 0) {
                    $residual_scenarios = $residual_scenarios->reject($scenario);
                }
            }
        }

        return $residual_scenarios->map(function (Scenario $scenario) {
            return $scenario->dangers;
        })->flatten(1)->unique('id');
    }
}
