<?php

namespace App\Repository;

use App\Calculator\MaturityAnswerLevelCalculator;
use App\Exceptions\MissingMaturityAnswerException;
use App\Models\Evaluation;
use App\Models\EvaluationMaturityAnswer;
use App\Models\MaturityAnswer;
use App\Models\MaturityQuestion;
use Illuminate\Support\Collection;

class EvaluationRepository
{
    private MaturityAnswerLevelCalculator $calculator;

    public function __construct(MaturityAnswerLevelCalculator $calculator)
    {
        $this->calculator = $calculator;
    }

    public function saveAutomaticAnswers($evaluation)
    {
        $questions = MaturityQuestion::where('auto', true)->get();
        foreach ($questions as $question) {
            // Calculate answer based on data
            // For "execution"
            // Get completed evaluation before this one
            // If there are none then all calculated levels are 0
            // (nombre de mesure basculée en mesure en place dans l'évaluation en cours/nombre de mesure prévue dans évaluation précédente) *100
            // 0 => level 0
            // 0 < calculated_level < 20 => level 1
            // 20 < calculated_level < 80 => level 2
            // calculated_level >= 80 => level 3
            // For "fondamentaux"
            // MOYENNE.INF( niveau de la mesure en place sauvegarde, niveau de la mesure en place mot de passe, niveau de la mesure en place mise à jour , niveau de la mesure en place sensibilisation)

            $calculated_level = 0;
            if ('Fondamentaux' === $question->title) {
                $calculated_level = $this->calculator->calculateFundamentalLevel($evaluation);
            }

            if ('Exécution' === $question->title) {
                // Get previous completed evaluation
                /**
                 * @var Evaluation|null $previousEvaluation
                 */
                $previousEvaluation = Evaluation::where('status', Evaluation::STATUS_DONE)
                    ->whereDate('updated_at', '<', $evaluation->updated_at->toDateString())
                    ->orderBy('updated_at', 'DESC')
                    ->first();
                $calculated_level = $this->calculator->calculateExecutionLevel($evaluation, $previousEvaluation);
            }

            $answer = MaturityAnswer::where('maturity_question_id', $question->id)
                ->where('level', $calculated_level)
                ->first();

            $evalAnswer = EvaluationMaturityAnswer::where('evaluation_id', $evaluation->id)
                ->where('maturity_question_id', $question->id)
                ->first();
            if (!$evalAnswer) {
                $evalAnswer = new EvaluationMaturityAnswer();
            }

            $evalAnswer->maturity_question_id = $question->id;
            $evalAnswer->maturity_answer_id = $answer->id;
            $evalAnswer->evaluation_id = $evaluation->id;
            $evalAnswer->save();
        }
    }

    /**
     * Save maturity answers.
     *
     * @throws MissingMaturityAnswerException
     */
    public function saveMaturity(Evaluation $evaluation, Collection $answers, int $draft = 0)
    {
        $questions = MaturityQuestion::where('auto', false)->get();
        foreach ($questions as $question) {
            // Get answer from sent values
            $answer = $answers->filter(function ($a) use ($question) {
                return $a['maturity_question_id'] === $question->id;
            })->first();
            if ((!isset($answer) || !isset($answer['maturity_answer_id'])) && 0 === $draft) {
                throw new MissingMaturityAnswerException();
            }

            $maturity = isset($answer['id']) ? EvaluationMaturityAnswer::find($answer['id']) : new EvaluationMaturityAnswer();
            $maturity->maturity_question_id = $answer['maturity_question_id'];
            $maturity->maturity_answer_id = $answer['maturity_answer_id'];
            $maturity->evaluation_id = $evaluation->id;
            $maturity->save();
        }

        $this->saveAutomaticAnswers($evaluation);
    }
}
